#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "adc.h"


// We iterate through all channels in free running mode
volatile uint8_t adc_chan;
volatile uint16_t adc_ss_sum; // Current conversion sum
volatile uint8_t adc_ss_count; // Current number of conversions per channel

volatile uint16_t adc_voltage;
volatile uint8_t adc_voltage_new;
volatile uint16_t adc_current;
volatile uint8_t adc_current_new;
volatile uint16_t adc_battery_voltage;
volatile uint8_t adc_battery_voltage_new;
volatile uint16_t adc_load_current;
volatile uint8_t adc_load_current_new;


void adc_init(void)
{
    // ADMUX – ADC Multiplexer Selection Register
    ADMUX = (0 << REFS1) | (0 << REFS0) // External reference;
          | (0 << ADLAR); // Right adjust result

    // ADCSRA – ADC Control and Status Register A
    ADCSRA = (1 << ADIE) // Interrupt enabled
           | ADC_PRESCALER_REG;

    // ADCSRB – ADC Control and Status Register B
    ADCSRB = 0;

    // Settings for first conversion
    adc_ss_sum = 0;
    adc_ss_count = 0;
    adc_chan = ADC_CHANNEL_PANEL_VOLTAGE;
    adc_set_channel(adc_chan);


    adc_voltage = 0;
    adc_voltage_new = 0;
    adc_current = 0;
    adc_battery_voltage = 0;
    adc_load_current = 0;
}


ISR(ADC_vect)
{
    adc_ss_sum += ADCW;
    adc_ss_count++;

    if (adc_ss_count >= ADC_SS_COUNT)
    {
        // Move supersampling result to variable
        uint16_t v = adc_ss_sum / ADC_SS_DIV;
        switch (adc_chan)
        {
            case ADC_CHANNEL_PANEL_VOLTAGE:
                adc_voltage = v;
                adc_voltage_new = 1;
                adc_chan = ADC_CHANNEL_PANEL_CURRENT;
                break;

            case ADC_CHANNEL_PANEL_CURRENT:
                adc_current = v;
                adc_current_new = 1;
                adc_chan = ADC_CHANNEL_LOAD_CURRENT;
                break;

            case ADC_CHANNEL_LOAD_CURRENT:
                adc_load_current = v;
                adc_load_current_new = 1;
                adc_chan = ADC_CHANNEL_BATT_VOLTAGE;
                break;

            case ADC_CHANNEL_BATT_VOLTAGE:
                adc_battery_voltage = v;
                adc_battery_voltage_new = 1;
                adc_chan = ADC_CHANNEL_PANEL_VOLTAGE;
                break;

            default:
                break;
                // void (calm compiler)
        }
        // Set next channel
        adc_ss_sum = 0;
        adc_ss_count = 0;
        adc_set_channel(adc_chan);
    }

    adc_start_conversion();
}


//// Single conversion
//uint16_t adc_conversion(uint8_t channel)
//{
//    adc_setChannel(channel);
//    return _adc_conversion();
//}


//// 16x supersampling to achieve 12 bit resolution with 10 bit ADC
//uint16_t adc_12BitConversion(uint8_t channel)
//{
//    uint16_t adc = 0;
//    adc_setChannel(channel);
//    for (uint8_t i = 16; i; i--)
//        adc += _adc_conversion();
//    return adc / 4;
//}



