#ifndef _LCD_HD44780_H__
#define _LCD_HD44780_H__

/*
Interface HD44780 compatible lcd-controller.

8-bit-mode:
    Using two ports on controller

pcf8574-mode:
    4 bit mode using i2c portexpander

Last Change: 2015-04-17 (Frank Baettermann)
*/

#include <stdint.h>
#include <avr/pgmspace.h>

//#define HD44780_MODE_8BIT
#define HD44780_MODE_PCF8574


#ifdef HD44780_MODE_8BIT
// Pin-configuration
#define HD44780_PORT    PORTD
#define HD44780_PIN     PIND
#define HD44780_DDR     DDRD
#define HD44780_RS      PD5
#define HD44780_RW      PD6
#define HD44780_EN      PD7

// data-port
#define HD44780_DPORT   PORTC
#define HD44780_DPIN    PINC
#define HD44780_DDDR    DDRC
#endif //HD44780_MODE_8BIT

#ifdef HD44780_MODE_PCF8574
#include "pcf8574.h"

#define HD44780_PCF8574_I2CADDR    (PCF8574_I2CADDR + 0 + 0 + 0) // A0,A1,A2 low

// Mapping of port expander pins to LCD pins
#define HD44780_PCF8574_DB4     (1 << 4)
#define HD44780_PCF8574_DB5     (1 << 5)
#define HD44780_PCF8574_DB6     (1 << 6)
#define HD44780_PCF8574_DB7     (1 << 7)
#define HD44780_PCF8574_RS      (1 << 0)
#define HD44780_PCF8574_WR      (1 << 1)
#define HD44780_PCF8574_EN      (1 << 2)
#define HD44780_PCF8574_BL      (1 << 3) // Backlight


int8_t hd44780_pcf8574_write(uint8_t d);
void hd44780_pcf8574_backlight_on(void);
void hd44780_pcf8574_backlight_off(void);

#endif //HD44780_MODE_PCF8574

// LCD commands
#define HD44780_CMD_FS_4BIT         (0 << 4)
#define HD44780_CMD_FS_8BIT         (1 << 4)
#define HD44780_CMD_FS_1LINE        (0 << 3)
#define HD44780_CMD_FS_2LINE        (1 << 3) // (per controller, 4 lines need 2 controllers)
#define HD44780_CMD_FS_5X7FONT      (0 << 2)
#define HD44780_CMD_DC_ENABLE       (1 << 2)
#define HD44780_CMD_DC_CURSOR       (1 << 1)
#define HD44780_CMD_DC_CURSORBLINK  (1 << 0)
#define HD44780_CMD_CS_CURSORLEFT   (0 << 2)
#define HD44780_CMD_CS_CURSORRIGHT  (1 << 2)
#define HD44780_CMD_CLEAR_DISPLAY   0x01
#define HD44780_CMD_CURSOR_HOME     0x02

#define HD44780_USERCHAR_LENGTH     8 // 7 for 5x7, 8 for 5x8


int8_t hd44780_init(void);
void hd44780_enablePulse(void);
void hd44780_write_cmd(char command);
void hd44780_clear(void);
void hd44780_putc(char data);
//void hd44780_puts(char* s);
//void hd44780_puts_P(const char *s);
void hd44780_busyLoop(void);
void hd44780_set_cursor(uint8_t x, uint8_t y);
void hd44780_define_char(uint8_t no, uint8_t* data);

#endif
