#ifndef _I2C_H__
#define _I2C_H__

#include <stdint.h>
#include <util/twi.h>


#define I2C_CLOCK           100 // [kHz]
#define I2C_PRESCALER1      ((0 << TWPS1) | (0 << TWPS0))
#define I2C_PRESCALER4      ((0 << TWPS1) | (1 << TWPS0))
#define I2C_PRESCALER16     ((1 << TWPS1) | (0 << TWPS0))
#define I2C_PRESCALER64     ((1 << TWPS1) | (1 << TWPS0))

// Calculate smallest possible prescaler (assume TWBR is 255)
#define I2C_PRESCALER_OPT   ((F_CPU / 1000 / I2C_CLOCK - 16) / 510) // /510 == /(2*TWBR_MAX)


#if (I2C_PRESCALER_OPT <= 1)
    // Everything < 1... rounds to zero
    #define I2C_PRESCALER   I2C_PRESCALER1
    #define I2C_TWBR        (((F_CPU / 2000UL) / I2C_CLOCK - 8) / 1)
#elif (I2C_PRESCALER_OPT <= 5)
    // 4.8 would round to 4.0 (would result in too big TWBR)
    #define I2C_PRESCALER   I2C_PRESCALER4
    #define I2C_TWBR        (((F_CPU / 2000UL) / I2C_CLOCK - 8) / 4)
#elif (I2C_PRESCALER_OPT <= 17)
    #define I2C_PRESCALER   I2C_PRESCALER16
    #define I2C_TWBR        (((F_CPU / 2000UL) / I2C_CLOCK - 8) / 16)
#elif (I2C_PRESCALER_OPT <= 65)
    #define I2C_PRESCALER   I2C_PRESCALER64
    #define I2C_TWBR        (((F_CPU / 2000UL) / I2C_CLOCK - 8) / 64)
#else
    #error "Cannot select appropriate i2c prescaler."
#endif



void i2c_init(void);
//uint8_t i2c_init(uint16_t clock_kHz);
int8_t i2c_transaction(uint8_t i2c_address, uint8_t w_length, uint8_t *w_data, uint8_t r_length, uint8_t *r_data);
uint8_t i2c_write(uint8_t data);

uint8_t i2c_start(void);
uint8_t i2c_repeatedStart(void);
uint8_t i2c_stop(void);
uint8_t i2c_address(uint8_t address, uint8_t read);
uint8_t i2c_writeByte(uint8_t data);
uint8_t i2c_writeData(uint8_t *data, uint8_t length);
uint8_t i2c_readByte(uint8_t *data);
uint8_t i2c_readData(uint8_t *data, uint8_t length);

#endif
