#ifndef _LM75_H__
#define _LM75_H__

#include <stdint.h>

#define LM75_ZERODEGCINDEGK     27315 // Zero deg C in deg K (*100)
#define LM75_ERRVAL_CELSIUS     INT16_MAX
#define LM75_ERRVAL_KELVIN      UINT16_MAX

#define LM75_I2CADDR            0x90
#define LM75_REG_TEMPERATURE    0x00
#define LM75_REG_CONFIGURATION  0x01
#define LM75_REG_TLOW           0x02
#define LM75_REG_THIGH          0x03

#define LM75_CONF_SHUTDOWN      0x01
#define LM75_CONF_THERMOSTAT    0x02
#define LM75_CONF_POLARITY      0x04
#define LM75_CONF_FAULTQUEUE1   0x00
#define LM75_CONF_FAULTQUEUE2   0x08
#define LM75_CONF_FAULTQUEUE4   0x10
#define LM75_CONF_FAULTQUEUE6   0x18

#define TMP275_CONF_RES09       0x00
#define TMP275_CONF_RES10       0x20
#define TMP275_CONF_RES11       0x40
#define TMP275_CONF_RES12       0x60
#define TMP275_CONF_ONESHOT     0x80

// Define the following if you need access to all registers
// #define LM75_ALLREGISTERS


int8_t lm75_read_temperature(uint8_t i2caddr, uint8_t *data);
int16_t lm75_read_temperature_celsius(uint8_t i2caddr);
uint16_t lm75_read_temperature_kelvin(uint8_t i2caddr);
int8_t lm75_read_configuration(uint8_t i2caddr, uint8_t *data);
int8_t lm75_write_configuration(uint8_t i2caddr, uint8_t config);

#ifdef LM75_ALLREGISTERS
int8_t lm75_read_tlow(uint8_t i2caddr, uint8_t *temp);
int8_t lm75_write_tlow(uint8_t i2caddr, uint8_t *temp);
int8_t lm75_read_thigh(uint8_t i2caddr, uint8_t *temp);
int8_t lm75_write_thigh(uint8_t i2caddr, uint8_t *temp);
#endif // LM75_ALLREGISTERS

#endif // _LM75_H__
