#ifndef _MAIN_H__
#define _MAIN_H__

#include <stdint.h>
#include "pcf8574.h"
#include "time.h"

#define STARTUP_TEXT1       "Mobile charging"
#define STARTUP_TEXT2       "  station (v1) "
#define FIRMWARE_VERSION    1

#define MAIN_POWERCTRL_INTERVAL 2 // [s]

// Battery voltages
#define BATT_VOLTAGE_STOPCHARGE         13800 // 2.30V/cell
#define BATT_VOLTAGE_RECHARGE           13200 // 2.20V/cell
#define BATT_VOLTAGE_LOAD_DROP          11100 // 1.85V/cell
#define BATT_VOLTAGE_LOAD_CONNECT       12050 // 2.01V/cell
#define BATT_VOLTAGE_LIMPINGMODE_ENTER  11700 // 1.95V/cell
#define BATT_VOLTAGE_LIMPINGMODE_DROP1  11400 // 1.90V/cell
#define BATT_VOLTAGE_LIMPINGMODE_STOP   12000 // 2.00V/cell, should be lower than BATT_VOLTAGE_LOAD_CONNECT

#define CSV_DELIMITER   ';'

#define CHRG_SLOTS      8
#define CHRG_TIMEOUT    7200 // [s]

#define LCD_ROWS                2
#define LCD_COLS                16

#define LCD_MESSAGE_TIMEOUT     4 // [s]
#define LCD_BACKLIGHT_TIMEOUT   6 // [s]
#define LCD_CUSTOMCHARS_STATUS  3
#define LCD_CHAR_SUN            0
#define LCD_CHAR_BATTERY        1
#define LCD_CHAR_DEGC           2

#define AUIR3315_CURRENTRATIO   2800UL
#define AUIR3315_PANEL_RIFB     1500UL // [Ohms]
#define AUIR3315_LOAD_RIFB      1200UL // [Ohms]
#define AUIR3325_INRUSH_DELAY   50 // [ms]

#define IO_CHARGEREN_DDR        DDRB
#define IO_CHARGEREN_PORT       PORTB
#define IO_CHARGEREN_BIT        PB2
#define IO_CHARGEREN_SET        IO_CHARGEREN_PORT |= (1 << IO_CHARGEREN_BIT)
#define IO_CHARGEREN_UNSET      IO_CHARGEREN_PORT &= ~(1 << IO_CHARGEREN_BIT)
#define IO_ALLOUTEN_DDR         DDRB
#define IO_ALLOUTEN_PORT        PORTB
#define IO_ALLOUTEN_BIT         PB3
#define IO_ALLOUTEN_SET         IO_ALLOUTEN_PORT |= (1 << IO_ALLOUTEN_BIT)
#define IO_ALLOUTEN_UNSET       IO_ALLOUTEN_PORT &= ~(1 << IO_ALLOUTEN_BIT)
#define IO_ALLOUTINRUSH_DDR     DDRB
#define IO_ALLOUTINRUSH_PORT    PORTB
#define IO_ALLOUTINRUSH_BIT     PB4
#define IO_ALLOUTINRUSH_SET     IO_ALLOUTINRUSH_PORT |= (1 << IO_ALLOUTINRUSH_BIT)
#define IO_ALLOUTINRUSH_UNSET   IO_ALLOUTINRUSH_PORT &= ~(1 << IO_ALLOUTINRUSH_BIT)
#define IO_LEDFULL_DDR          DDRB
#define IO_LEDFULL_PORT         PORTB
#define IO_LEDFULL_BIT          PB5
#define IO_LEDFULL_SET          IO_LEDFULL_PORT |= (1 << IO_LEDFULL_BIT)
#define IO_LEDFULL_UNSET        IO_LEDFULL_PORT &= ~(1 << IO_LEDFULL_BIT)

#define IO_CHAN00_DDR       DDRD
#define IO_CHAN00_PORT      PORTD
#define IO_CHAN00_BIT       PD6
#define IO_CHAN01_DDR       DDRD
#define IO_CHAN01_PORT      PORTD
#define IO_CHAN01_BIT       PD2
#define IO_CHAN02_DDR       DDRD
#define IO_CHAN02_PORT      PORTD
#define IO_CHAN02_BIT       PD7
#define IO_CHAN03_DDR       DDRD
#define IO_CHAN03_PORT      PORTD
#define IO_CHAN03_BIT       PD3
#define IO_CHAN04_DDR       DDRB
#define IO_CHAN04_PORT      PORTB
#define IO_CHAN04_BIT       PB0
#define IO_CHAN05_DDR       DDRD
#define IO_CHAN05_PORT      PORTD
#define IO_CHAN05_BIT       PD4
#define IO_CHAN06_DDR       DDRB
#define IO_CHAN06_PORT      PORTB
#define IO_CHAN06_BIT       PB1
#define IO_CHAN07_DDR       DDRD
#define IO_CHAN07_PORT      PORTD
#define IO_CHAN07_BIT       PD5

#define BUTTON_INTERVAL         24 // [ms]
#define BUTTON_PCF8574_I2CADDR  (PCF8574_I2CADDR + 2) // A0 high, A1 low, A2 low

typedef struct
{
    uint8_t enabled : 1, // Channel available (can be activated)
            active  : 1; // Channel is currently active
    uint16_t timeout;
} chrg_slot_t;

typedef struct
{
    uint8_t
        load_connected : 1,
        panel_connected : 1,
        limping_mode : 1,
        reserved : 3,
        led_status : 1,
        temperature_initialized: 1;
} status_t;

typedef struct
{
    uint8_t data[LCD_ROWS][LCD_COLS];
    uint8_t cursorx;
    uint8_t cursory;
} lcdbuffer_t;


void lcd_draw(void);
void lcd_flush(void);
void lcd_clear(void);void lcd_set_cursor(uint8_t x, uint8_t y);
void lcd_putc(char c);
void lcd_puts(char *s);
void lcd_puts_P(const char *s);void uart_print_header(void);
void uart_print(void);
void lcd_custom_chars_status(void);
void lcd_status(void);
void lcd_msg_load_dropped(void);
void lcd_msg_load_connected(void);
void lcd_msg_limping_started(void);
void lcd_msg_limping_stopped(void);
void lcd_msg_limping_slotdisable(uint8_t i);
void lcd_msg_limping_na(void);
void lcd_msg_batterylow_na(void);
void lcd_msg_slot_enabled(uint8_t i);
void lcd_msg_slot_timedout(uint8_t i);
void lcd_msg_timers(void);
void lcd_backlight(void);
void slot_enable(uint8_t i);
void slot_disable(uint8_t i);
uint8_t slots_active(void);
uint8_t slot_longestactive(void);
void main_init(void);
void main_buttons(void);
void main_measurements(void);
void main_measurements_temperature(void);
void main_powerctrl(void);
void main_lcd(void);
int main(void);

#endif

