#include <stdint.h>
#include "pcf8574.h"
#include "i2c.h"


/*
Interface PCF8574 port expander (open drain outputs).

To use as input write corresponding bit "high" and read.
Use pullup if internal 100µA current sources are not sufficient.
Maximum frequency is 100 kHz.

Last change: 2015-04-16 (Frank Baettermann)
*/

/*
 * Write byte to port expander.
 * Returns: 0 if write is successful, -1 otherwise.
 */
int8_t pcf8574_write(uint8_t addr, uint8_t data)
{
    int8_t ret = 0;
    if (i2c_start() || i2c_address(addr, 0) || i2c_writeByte(data))
        ret = -1;
    i2c_stop();
    return ret;
}


/*
 * Read byte from port expander.
 * Returns: 0 if write is successful, -1 otherwise.
 */
int8_t pcf8574_read(uint8_t addr, uint8_t* data)
{
    int8_t ret = 0;
    if (i2c_start() || i2c_address(addr, 1) || i2c_readData(data, 1))
        ret = -1;
    i2c_stop();
    return ret;
}

