#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <util/atomic.h>
#include "main.h"
#include "time.h"

volatile runtime_t runtime;
volatile uint16_t time_ms; // [ms] overflows!

// Variables used by others to detect delays.
// They are decremented by one after the interval.
volatile uint8_t time_delay_uart_s = 0;
volatile uint8_t time_delay_lcdbl_s = 0;
volatile uint8_t time_delay_lcdmsg_s = 0;
volatile uint16_t time_delay_powerctrl_s = 0;
volatile uint8_t time_delay_button_ms = 0;
volatile uint8_t time_delay_250ms = 0;
volatile uint8_t time_delay_loadinrush_ms = 0;

ISR(TIMER0_COMPA_vect)
{
    time_ms += TIME_INTERVAL_MS;
    runtime.ms  += TIME_INTERVAL_MS;
    if (runtime.ms >= 1000)
    {
        runtime.ms -= 1000;
        runtime.s++;

        if (time_delay_uart_s)
            time_delay_uart_s--;
        if (time_delay_lcdbl_s)
            time_delay_lcdbl_s--;
        if (time_delay_lcdmsg_s)
            time_delay_lcdmsg_s--;
        if (time_delay_powerctrl_s)
            time_delay_powerctrl_s--;
    }

    time_delay_button_ms = time_delay_button_ms > TIME_INTERVAL_MS ? time_delay_button_ms - TIME_INTERVAL_MS : 0;
    time_delay_250ms = time_delay_250ms > TIME_INTERVAL_MS ? time_delay_250ms - TIME_INTERVAL_MS : 0;
    time_delay_loadinrush_ms = time_delay_loadinrush_ms > TIME_INTERVAL_MS ? time_delay_loadinrush_ms - TIME_INTERVAL_MS : 0;
}

void time_init(void)
{
    time_ms = 0;
    runtime.s = 0;
    runtime.ms = 0;

    // No outout, no pwm; Clear Timer on Compare Match (CTC) Mode
    TCCR0A = (1 << WGM01);
    // Prescaler
    TCCR0B = TIME_TIMER0_CLK;
    // Top value
    OCR0A = TIME_TIMER_TOP;
    // Enable output compare 1a interrupt
    TIMSK0 = (1 << OCIE0A);
}

/*
 * Atomic access to runtime
 */
void time_get_runtime(runtime_t *rt)
{
    ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
    {
        rt->s = runtime.s;
        rt->ms = runtime.ms;
    }
}

//
///*
// * Convert runtime to string: "s.SSS"
// */
//char* time_runtime_str(runtime_t* rt, char* buf)
//{
//    uint8_t len;
//
//    ultoa(rt->s, buf, 10);
//    len = strlen(buf);
//    buf[len++] = '.';
//    if (rt->ms < 100)
//        buf[len++] = '0';
//    if (rt->ms < 10)
//        buf[len++] = '0';
//    utoa(rt->ms, &(buf[len]), 10);
//
//    return buf;
//}
//
///*
// * Add rt2 to rt1.
// */
//runtime_t time_runtime_add(runtime_t* rt1, runtime_t* rt2)
//{
//    runtime_t ret;
//    ret.s = rt1->s + rt2->s;
//    ret.ms = rt1->ms - rt2->ms;
//    if (ret.ms >= 1000)
//    {
//        ret.s++;
//        ret.ms -= 1000;
//    }
//    return ret;
//}
//
///*
// * Subtract rt2 from rt1.
// * Make sure rt1 > rt2!
// */
//runtime_t time_runtime_substract(runtime_t* rt1, runtime_t* rt2)
//{
//    runtime_t ret;
//    ret.s = rt1->s - rt2->s;
//    if (rt2->ms > rt1->ms)
//    {
//        ret.s--;
//        ret.ms = rt1->ms + 1000 - rt2->ms;
//    }
//    else
//        ret.ms = rt1->ms - rt2->ms;
//    return ret;
//}
