/*
 * Setup Timer0 (8 bit) as timebase
 *
 * Target: ATmega328P
 * Author: Frank Baettermann
 */
#ifndef _TIME_H__
#define _TIME_H__

#include <stdint.h>
#include <avr/io.h>

#define TIME_INTERVAL_MS    4 // [ms]

#define TIMER0_CLK_STOP     ((0 << CS02) | (0 << CS01) | (0 << CS00))
#define TIMER0_CLK_PS1      ((0 << CS02) | (0 << CS01) | (1 << CS00))
#define TIMER0_CLK_PS8      ((0 << CS02) | (1 << CS01) | (0 << CS00))
#define TIMER0_CLK_PS64     ((0 << CS02) | (1 << CS01) | (1 << CS00))
#define TIMER0_CLK_PS256    ((1 << CS02) | (0 << CS01) | (0 << CS00))
#define TIMER0_CLK_PS1024   ((1 << CS02) | (0 << CS01) | (1 << CS00))
#define TIMER0_CLK_T0FE     ((1 << CS02) | (1 << CS01) | (0 << CS00))
#define TIMER0_CLK_T0RE     ((1 << CS02) | (1 << CS01) | (1 << CS00))


#if (F_CPU == 16000000)
    #define TIME_TIMER_PRESCALER    256
    #define TIME_TIMER_TOP          (F_CPU / TIME_TIMER_PRESCALER / (1000UL / TIME_INTERVAL_MS) - 1)
    #define TIME_TIMER0_CLK         TIMER0_CLK_PS256
#else
    #define TIME_TIMER_PRESCALER    1
    #define TIME_TIMER_TOP          1
    #define TIME_TIMER0_CLK         TIMER0_CLK_STOP
    #error "Don't know how to setup the timebase."
#endif

#if ((TIME_TIMER_TOP + 1) * (1000 / TIME_INTERVAL_MS) * TIME_TIMER_PRESCALER != F_CPU)
    #error "Please select different values for the timebase interval and timer prescaler to eliminate the error in time calculation."
#endif

typedef struct
{
    uint32_t s;
    uint16_t ms;
} runtime_t;

// Make time accessible
extern volatile uint16_t time_ms; // [ms] overflows!
extern volatile uint8_t time_delay_uart_s;
extern volatile uint8_t time_delay_lcdbl_s;
extern volatile uint8_t time_delay_lcdmsg_s;
extern volatile uint16_t time_delay_powerctrl_s;
extern volatile uint8_t time_delay_button_ms;
extern volatile uint8_t time_delay_250ms;
extern volatile uint8_t time_delay_loadinrush_ms;

void time_init(void);
void time_get_runtime(runtime_t *rt);
//char* time_runtime_str(runtime_t* rt, char* buf);
//runtime_t time_runtime_add(runtime_t* rt1, runtime_t* rt2);
//runtime_t time_runtime_substract(runtime_t* rt1, runtime_t* rt2);

#endif
