#ifndef _UART_H__
#define _UART_H__

#include <avr/io.h>
#include <stdint.h>

/*
Interrupt controlled interface to hardware UART0.

With help from:
- http://www.rn-wissen.de/index.php/UART_mit_avr-gcc
- http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/Der_UART
*/


// Baudrate register calculations
#define UART_BAUD       9600UL
#define UART_UBRR_VAL   ((F_CPU + UART_BAUD * 8) / (UART_BAUD * 16) - 1)  // clever runden
#define UART_BAUD_REAL  (F_CPU / (16 * (UART_UBRR_VAL + 1)))    // Reale Baudrate
#define UART_BAUD_ERROR ((UART_BAUD_REAL * 1000) / UART_BAUD)   // Fehler in Promille, 1000 = kein Fehler.
#if ((UART_BAUD_ERROR < 985) || (UART_BAUD_ERROR > 1015))
    #error "Baudrate error > 1.5%"
#endif

#define UART_BUFFER_RX  16
#define UART_BUFFER_TX  112


void uart_init(void);
int16_t uart_getc(void);
uint8_t uart_getc_wait(void);
void uart_putc(const char c);
void uart_puts(char *s);
void uart_put(uint8_t *a, uint8_t len);
void uart_puts_P(const char *s);


// Wait until all data is sent.
static inline void uart_flush(void)
{
    while (UCSR0B & (1 << UDRIE0)) {}
}

#endif

